==> adeptgenerator-8.0.0%2B1.tar.gz: lib/src/base_generator_for_annotation.dart <==
import 'package:analyzer/dart/constant/value.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:source_gen/source_gen.dart';

import 'package:adeptgenerator/src/utils/utils.dart';
import 'package:adeptgenerator/src/utils/shared_checkers.dart';
import 'package:adeptgenerator/src/utils/value_literal_generator.dart';

abstract class BaseGeneratorForAnnotation<A, K>
    extends GeneratorForAnnotation<A> {
  final annotationChecker = TypeChecker.fromRuntime(A);
  final keyAnnotationChecker = TypeChecker.fromRuntime(K);

  /// Check if the classElement has a default empty constructor like MyClass()
  checkDefaultConstructor(ClassElement classElement,
      {String annotationName = 'this annotation'}) {
    bool hasDefaultConstructor = false;
    for (var constructor in classElement.constructors) {
      if (constructor.isDefaultConstructor) {
        hasDefaultConstructor = true;
        continue;
      }
    }
    if (!hasDefaultConstructor) {
      throw InvalidGenerationSourceError(
          'To use $annotationName make sure '
          'to have a default constructor in your class '
          '${classElement.displayName}.',
          todo: 'Add a datult constructor like MyClass() or '
              'MyClass({String param1, String param2}) to your '
              '${classElement.displayName} or remove the $annotationName '
              'annotation from it.',
          element: classElement);
    }
  }

  /// Get ClassElement from generic Element if possible, otherwise throws
  /// exception
  ClassElement getClassElement(Element element,
      {String annotationName = 'this annotation'}) {
    if (element is! ClassElement) {
      throw InvalidGenerationSourceError(
          'Generator cannot target '
          '${element.displayName}. $annotationName can only be applied to a '
          'class.',
          todo: 'Check the type of ${element.displayName} or remove the '
              'annotation from it.',
          element: element);
    }
    return element;
  }

  /// Returns a literal value for [dartObject] if possible, otherwise throws
  /// an [InvalidGenerationSourceError] using [typeInformation] to describe
  /// the unsupported type.
  Object? literalForObject(
    FieldElement element,
    DartObject dartObject,
    Iterable<String> typeInformation,
  ) {
    if (dartObject.isNull) {
      return null;
    }

    final reader = ConstantReader(dartObject);

    String? badType;
    if (reader.isSymbol) {
      badType = 'Symbol';
    } else if (reader.isType) {
      badType = 'Type';
    } else if (dartObject.type is FunctionType) {
      // TODO: Support calling function for the default value?
      badType = 'Function';
    } else if (!reader.isLiteral) {
      badType = dartObject.type!.element!.name;
    }

    if (badType != null) {
      badType = typeInformation.followedBy([badType]).join(' > ');
      throw InvalidGenerationSourceError(
          '`defaultValue` is `$badType`, it must be a literal.',
          element: element);
    }

    if (reader.isDouble || reader.isInt || reader.isString || reader.isBool) {
      return reader.literalValue;
    }

    if (reader.isList) {
      return [
        for (var e in reader.listValue)
          literalForObject(element, e, [
            ...typeInformation,
            'List',
          ])
      ];
    }

    if (reader.isSet) {
      return {
        for (var e in reader.setValue)
          literalForObject(element, e, [
            ...typeInformation,
            'Set',
          ])
      };
    }

    if (reader.isMap) {
      final mapTypeInformation = [
        ...typeInformation,
        'Map',
      ];
      return reader.mapValue.map(
        (k, v) => MapEntry(
          literalForObject(element, k!, mapTypeInformation),
          literalForObject(element, v!, mapTypeInformation),
        ),
      );
    }

    badType = typeInformation.followedBy(['$dartObject']).join(' > ');

    InvalidGenerationSourceError(
      'The provided value is not supported: $badType. '
      'This may be an error in package:adeptgenerator. '
      'Please rerun your build with `--verbose` and file an issue.',
      element: element,
    );

    return null;
  }

  /// Returns a literal object representing the value of [fieldName] in [obj].
  ///
  /// If [mustBeEnum] is `true`, throws an [InvalidGenerationSourceError] if
  /// either the annotated field is not an `enum` or `List` or if the value in
  /// [fieldName] is not an `enum` value.
  Object? annotationValue(
      DartObject annotation, FieldElement element, String fieldName,
      {bool mustBeEnum = false}) {
    final obj = ConstantReader(annotation);
    final annotationValue = obj.read(fieldName);

    final enumFields = annotationValue.isNull
        ? null
        : iterateEnumFields(annotationValue.objectValue.type!);
    if (enumFields != null) {
      if (mustBeEnum) {
        late DartType targetEnumType;
        if (isEnum(element.type)) {
          targetEnumType = element.type;
        } else if (coreIterableTypeChecker.isAssignableFromType(element.type)) {
          targetEnumType = coreIterableGenericType(element.type);
        } else {
          throwUnsupported(
            element,
            '`$fieldName` can only be set on fields of type enum or on '
            'Iterable, List, or Set instances of an enum type.',
          );
        }
        final annotatedEnumType = annotationValue.objectValue.type!;
        if (!interfaceTypesEqual(annotatedEnumType, targetEnumType)) {
          throwUnsupported(
            element,
            '`$fieldName` has type '
            '`${targetEnumType.getDisplayString(withNullability: false)}`, but '
            'the provided unknownEnumValue is of type '
            '`${annotatedEnumType.getDisplayString(withNullability: false)}`.',
          );
        }
      }

      final enumValueNames =
          enumFields.map((p) => p.name).toList(growable: false);

      final enumValueName = enumValueForDartObject<String>(
          annotationValue.objectValue, enumValueNames, (n) => n);

      return '${annotationValue.objectValue.type!.element!.name}.$enumValueName';
    } else {
      final defaultValueLiteral = annotationValue.isNull
          ? null
          : literalForObject(element, annotationValue.objectValue, []);
      if (defaultValueLiteral == null) {
        return null;
      }
      if (mustBeEnum) {
        throwUnsupported(
          element,
          'The value provided for `$fieldName` must be a matching enum.',
        );
      }
      return valueLiteralAsDart(defaultValueLiteral);
    }
  }
}

==> adeptgenerator-9.0.0%2B1.tar.gz: lib/src/base_generator_for_annotation.dart <==
import 'package:analyzer/dart/constant/value.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:source_gen/source_gen.dart';

import 'package:adeptgenerator/src/utils/utils.dart';
import 'package:adeptgenerator/src/utils/shared_checkers.dart';
import 'package:adeptgenerator/src/utils/value_literal_generator.dart';

abstract class BaseGeneratorForAnnotation<A, K>
    extends GeneratorForAnnotation<A> {
  final annotationChecker = TypeChecker.fromRuntime(A);
  final keyAnnotationChecker = TypeChecker.fromRuntime(K);

  ///Recursive "postOrder" visitor to ensure fields analysis in a inheritance logical order
  void superTypeVisitor(
      InterfaceType? iType, void fieldAnalyzer(FieldElement field)) {
    if (iType == null || iType.isDartCoreObject) return;
    superTypeVisitor(iType.element.supertype, fieldAnalyzer);
    iType.element.fields.forEach(fieldAnalyzer);
  }

  /// Check if the classElement has a default empty constructor like MyClass()
  void checkDefaultConstructor(ClassElement classElement,
      {String annotationName = 'this annotation'}) {
    bool hasDefaultConstructor = classElement.constructors
        .any((constructor) => constructor.isDefaultConstructor);
    if (!hasDefaultConstructor) {
      throw InvalidGenerationSourceError(
          'To use $annotationName make sure '
          'to have a default constructor in your class '
          '${classElement.displayName}.',
          todo: 'Add a default constructor like MyClass() or '
              'MyClass({String param1, String param2}) to your '
              '${classElement.displayName} or remove the $annotationName '
              'annotation from it.',
          element: classElement);
    }
  }

  /// Get ClassElement from generic Element if possible, otherwise throws
  /// exception
  ClassElement getClassElement(Element element,
      {String annotationName = 'this annotation'}) {
    if (element is! ClassElement) {
      throw InvalidGenerationSourceError(
          'Generator cannot target '
          '${element.displayName}. $annotationName can only be applied to a '
          'class.',
          todo: 'Check the type of ${element.displayName} or remove the '
              'annotation from it.',
          element: element);
    }
    return element;
  }

  /// Returns a literal value for [dartObject] if possible, otherwise throws
  /// an [InvalidGenerationSourceError] using [typeInformation] to describe
  /// the unsupported type.
  Object? literalForObject(
    FieldElement element,
    DartObject dartObject,
    Iterable<String> typeInformation,
  ) {
    if (dartObject.isNull) {
      return null;
    }

    final reader = ConstantReader(dartObject);

    String? badType;
    if (reader.isSymbol) {
      badType = 'Symbol';
    } else if (reader.isType) {
      badType = 'Type';
    } else if (dartObject.type is FunctionType) {
      // TODO: Support calling function for the default value?
      badType = 'Function';
    } else if (!reader.isLiteral) {
      badType = dartObject.type!.element!.name;
    }

    if (badType != null) {
      badType = typeInformation.followedBy([badType]).join(' > ');
      throw InvalidGenerationSourceError(
          '`defaultValue` is `$badType`, it must be a literal.',
          element: element);
    }

    if (reader.isDouble || reader.isInt || reader.isString || reader.isBool) {
      return reader.literalValue;
    }

    if (reader.isList) {
      return [
        for (var e in reader.listValue)
          literalForObject(element, e, [
            ...typeInformation,
            'List',
          ])
      ];
    }

    if (reader.isSet) {
      return {
        for (var e in reader.setValue)
          literalForObject(element, e, [
            ...typeInformation,
            'Set',
          ])
      };
    }

    if (reader.isMap) {
      final mapTypeInformation = [
        ...typeInformation,
        'Map',
      ];
      return reader.mapValue.map(
        (k, v) => MapEntry(
          literalForObject(element, k!, mapTypeInformation),
          literalForObject(element, v!, mapTypeInformation),
        ),
      );
    }

    badType = typeInformation.followedBy(['$dartObject']).join(' > ');

    InvalidGenerationSourceError(
      'The provided value is not supported: $badType. '
      'This may be an error in package:adeptgenerator. '
      'Please rerun your build with `--verbose` and file an issue.',
      element: element,
    );

    return null;
  }

  /// Returns a literal object representing the value of [fieldName] in [obj].
  ///
  /// If [mustBeEnum] is `true`, throws an [InvalidGenerationSourceError] if
  /// either the annotated field is not an `enum` or `List` or if the value in
  /// [fieldName] is not an `enum` value.
  Object? annotationValue(
      DartObject annotation, FieldElement element, String fieldName,
      {bool mustBeEnum = false}) {
    final obj = ConstantReader(annotation);
    final annotationValue = obj.read(fieldName);

    final enumFields = annotationValue.isNull
        ? null
        : iterateEnumFields(annotationValue.objectValue.type!);
    if (enumFields != null) {
      if (mustBeEnum) {
        late DartType targetEnumType;
        if (isEnum(element.type)) {
          targetEnumType = element.type;
        } else if (coreIterableTypeChecker.isAssignableFromType(element.type)) {
          targetEnumType = coreIterableGenericType(element.type);
        } else {
          throwUnsupported(
            element,
            '`$fieldName` can only be set on fields of type enum or on '
            'Iterable, List, or Set instances of an enum type.',
          );
        }
        final annotatedEnumType = annotationValue.objectValue.type!;
        if (!interfaceTypesEqual(annotatedEnumType, targetEnumType)) {
          throwUnsupported(
            element,
            '`$fieldName` has type '
            '`${targetEnumType.getDisplayString(withNullability: false)}`, but '
            'the provided unknownEnumValue is of type '
            '`${annotatedEnumType.getDisplayString(withNullability: false)}`.',
          );
        }
      }

      final enumValueNames =
          enumFields.map((p) => p.name).toList(growable: false);

      final enumValueName = enumValueForDartObject<String>(
          annotationValue.objectValue, enumValueNames, (n) => n);

      return '${annotationValue.objectValue.type!.element!.name}.$enumValueName';
    } else {
      final defaultValueLiteral = annotationValue.isNull
          ? null
          : literalForObject(element, annotationValue.objectValue, []);
      if (defaultValueLiteral == null) {
        return null;
      }
      if (mustBeEnum) {
        throwUnsupported(
          element,
          'The value provided for `$fieldName` must be a matching enum.',
        );
      }
      return valueLiteralAsDart(defaultValueLiteral);
    }
  }
}
